mod types;
use types::*;

static INPUT: &str = include_str!("input.txt");

fn decode_rps(c: char) -> RpsHand {
    match c {
        'A' => RpsHand::Rock,
        'X' => RpsHand::Rock,
        'B' => RpsHand::Paper,
        'Y' => RpsHand::Paper,
        'C' => RpsHand::Scissors,
        'Z' => RpsHand::Scissors,
        _ => panic!("invalid input"),
    }
}

fn decode_goal(c: char) -> Goal {
    match c {
        'X' => Goal::Lose,
        'Y' => Goal::Draw,
        'Z' => Goal::Win,
        _ => panic!("invalid input"),
    }
}

fn decode_plan(g: Goal, h: RpsHand) -> RpsHand {
    match g {
        Goal::Lose => h.wins_against(),
        Goal::Draw => h.ties_with(),
        Goal::Win => h.loses_to(),
    }
}

fn process_input(input: &str, conv_func: fn(a: char, b: char) -> Game) -> Vec<Game> {
    let mut games = vec![];
    for line in input.lines() {
        if !line.is_empty() {
            if line.len() != 3 {
                panic!("invalid input")
            } else {
                let a = line.chars().next().expect("invalid input");
                let b = line.chars().nth(2).expect("invalid input");
                games.push(conv_func(a, b));
            }
        }
    }
    games
}

fn conv_game(a: char, b: char) -> Game {
    Game {
        opponent: decode_rps(a),
        mine: decode_rps(b),
    }
}

fn conv_plan(a: char, b: char) -> Game {
    let opp = decode_rps(a);
    let goal = decode_goal(b);
    Game {
        opponent: opp,
        mine: decode_plan(goal, opp),
    }
}

fn score_shape(s: &RpsHand) -> i32 {
    match s {
        RpsHand::Rock => 1,
        RpsHand::Paper => 2,
        RpsHand::Scissors => 3,
    }
}

fn score_outcome(g: &Game) -> i32 {
    match (&g.mine, &g.opponent) {
        (RpsHand::Rock, RpsHand::Scissors) => 6,
        (RpsHand::Paper, RpsHand::Rock) => 6,
        (RpsHand::Scissors, RpsHand::Paper) => 6,
        _ => {
            if g.mine == g.opponent {
                3
            } else {
                0
            }
        }
    }
}

fn score_game(g: &Game) -> i32 {
    score_shape(&g.mine) + score_outcome(g)
}

pub fn solution_a() -> i32 {
    let games = process_input(INPUT, conv_game);
    games.iter().map(score_game).sum()
}

pub fn solution_b() -> i32 {
    let games = process_input(INPUT, conv_plan);
    games.iter().map(score_game).sum()
}
