use std::cmp::Ordering;

static INPUT: &str = include_str!("input.txt");

fn item_priority(c: char) -> u8 {
    if !c.is_ascii_alphabetic() {
        panic!("invalid item in rucksack: {}", c);
    }
    match c {
        'a'..='z' => (c as u8) - b'a' + 1,
        _ => (c as u8) - b'A' + 27,
    }
}

fn split_input_line(l: String) -> (Vec<char>, Vec<char>) {
    let n = l.len();
    if n % 2 != 0 {
        panic!("invalid line length");
    }
    let s = n / 2;
    let mut c1 = l[0..s].chars().collect::<Vec<_>>();
    c1.sort();
    c1.dedup();
    let mut c2 = l[s..n].chars().collect::<Vec<_>>();
    c2.sort();
    c2.dedup();
    (c1, c2)
}

fn parse_input_line(l: String) -> Vec<char> {
    let mut c1 = l.chars().collect::<Vec<_>>();
    c1.sort();
    c1.dedup();
    c1
}

fn find_items(c1: Vec<char>, c2: Vec<char>) -> Vec<char> {
    let mut results = vec![];
    let mut i1 = c1.iter().peekable();
    let mut i2 = c2.iter().peekable();
    while let (Some(a), Some(b)) = (i1.peek(), i2.peek()) {
        match a.cmp(b) {
            Ordering::Equal => {
                results.push(**a);
                i1.next();
                i2.next();
            }
            Ordering::Less => {
                i1.next();
            }
            Ordering::Greater => {
                i2.next();
            }
        }
    }
    results
}

pub fn solution_a() -> i32 {
    let input = INPUT.lines().map(String::from).map(split_input_line);
    input
        .into_iter()
        .map(|(a, b)| find_items(a, b))
        .map(|x| match x.first() {
            Some(y) => item_priority(*y) as i32,
            _ => panic!("no matches found"),
        })
        .sum()
}

pub fn solution_b() -> i32 {
    let mut total = 0;
    let mut input = INPUT.lines().map(String::from).map(parse_input_line);
    loop {
        let l1 = input.next();
        let l2 = input.next();
        let l3 = input.next();
        match (l1, l2, l3) {
            (Some(s1), Some(s2), Some(s3)) => {
                let intermediate_matches = find_items(s1, s2);
                let final_matches = find_items(s3, intermediate_matches);
                match final_matches.first() {
                    Some(x) => total += item_priority(*x) as i32,
                    _ => panic!("no matches found"),
                }
            }
            _ => break,
        }
    }
    total
}
