static INPUT: &str = include_str!("input.txt");

struct ElfRange {
    lower: i32,
    upper: i32,
}

fn split_ranges(s: &str) -> (ElfRange, ElfRange) {
    let binding = String::from(s)
        .split(|c| c == '-' || c == ',')
        .map(|c| c.parse().expect("not a number!"))
        .collect::<Vec<i32>>();
    let mut ranges = binding.iter();
    match (ranges.next(), ranges.next(), ranges.next(), ranges.next()) {
        (Some(a), Some(b), Some(c), Some(d)) => (
            ElfRange {
                lower: *a,
                upper: *b,
            },
            ElfRange {
                lower: *c,
                upper: *d,
            },
        ),
        _ => panic!("error parsing input"),
    }
}

fn total_overlap(a: ElfRange, b: ElfRange) -> bool {
    (a.lower <= b.lower && a.upper >= b.upper) || (b.lower <= a.lower && b.upper >= a.upper)
}

fn partial_overlap(a: ElfRange, b: ElfRange) -> bool {
    (b.lower >= a.lower && b.lower <= a.upper)
        || (b.upper >= a.lower && b.upper <= a.upper)
        || (a.lower >= b.lower && a.lower <= b.upper)
        || (a.upper >= b.lower && a.upper <= b.upper)
}

pub fn solution_a() -> i32 {
    let mut total = 0;
    for line in INPUT.lines() {
        let (a, b) = split_ranges(line);
        if total_overlap(a, b) {
            total += 1;
        }
    }
    total
}

pub fn solution_b() -> i32 {
    let mut total = 0;
    for line in INPUT.lines() {
        let (a, b) = split_ranges(line);
        if partial_overlap(a, b) {
            total += 1;
        }
    }
    total
}
