use std::collections::VecDeque;

static INPUT: &str = include_str!("input.txt");

type Instruction = (usize, usize, usize);

fn parse_input(s: &str) -> (Vec<VecDeque<char>>, Vec<Instruction>) {
    let mut initial: Vec<VecDeque<char>> = vec![];
    let mut instructions = vec![];
    let mut input = s.lines();
    for line in input.by_ref() {
        if line.is_empty() {
            break;
        }
        initial.push(line.chars().collect());
    }
    initial.reverse();
    initial.remove(0);
    let num_columns = (initial[0].len() + 1) / 4;
    let mut finals = vec![VecDeque::new(); num_columns];
    for line in initial {
        for (i, tower) in finals.iter_mut().enumerate().take(num_columns) {
            let ind = (i * 4) + 1;
            match line[ind] {
                ' ' => continue,
                _ => tower.push_front(line[ind]),
            }
        }
    }
    for line in input {
        let parsed_line = parse_instruction(line);
        instructions.push(parsed_line);
    }
    (finals, instructions)
}

fn parse_instruction(s: &str) -> Instruction {
    let binding = String::from(s)
        .split(|c: char| c == ' ' || c.is_alphabetic())
        .filter(|v| !v.is_empty())
        .map(|c| c.parse().expect("not a number!"))
        .collect::<Vec<usize>>();
    let mut instructions = binding.iter();
    match (
        instructions.next(),
        instructions.next(),
        instructions.next(),
    ) {
        (Some(a), Some(b), Some(c)) => (*a, *b, *c),
        _ => panic!("error parsing input"),
    }
}

fn execute_instructions_900(
    instructions: Vec<Instruction>,
    mut initial: Vec<VecDeque<char>>,
) -> Vec<VecDeque<char>> {
    for (num, src, dst) in instructions {
        for _i in 0..num {
            match initial[src - 1].pop_front() {
                Some(a) => initial[dst - 1].push_front(a),
                _ => panic!("attempted to pop from empty stack"),
            }
        }
    }
    initial
}

fn execute_instructions_9001(
    instructions: Vec<Instruction>,
    mut initial: Vec<VecDeque<char>>,
) -> Vec<VecDeque<char>> {
    for (num, src, dst) in instructions {
        for i in 0..num {
            match initial[src - 1].pop_front() {
                Some(a) => initial[dst - 1].insert(i, a),
                _ => panic!("attempted to pop from empty stack"),
            }
        }
    }
    initial
}

pub fn solution_a() -> String {
    let (initial, instructions) = parse_input(INPUT);
    let initial = execute_instructions_900(instructions, initial);
    let mut result = String::new();
    for tower in initial {
        result.push(tower[0]);
    }
    result
}

pub fn solution_b() -> String {
    let (initial, instructions) = parse_input(INPUT);
    let initial = execute_instructions_9001(instructions, initial);
    let mut result = String::new();
    for tower in initial {
        result.push(tower[0]);
    }
    result
}
